/*
 * ROX Mixer Applet version 0.1
 * Copyright (c) 2001-2003 Lubimov Alexey <avl@altlinux.ru>
 * port from suxpanel  Mixer Applet version 0.1 Copyright (c) 2001-2003 Leandro Pereira <leandro@linuxmag.com.br>
 *
 * This program may be distributed under the terms of GNU General
 * Public License version 2. You should have received a copy of the
 * license with this program; if not, please consult http://www.fsf.org/.
 *
 * This program comes with no warranty. Use at your own risk.
 *
 */

#include <gtk/gtk.h>
#include <gmodule.h>                                                                                                         
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

static GdkWindow *socket = NULL;
static GtkWidget *window;
static gint32 xid;

static GtkWidget *vol_wnd = NULL;
static gboolean is_inside_vol = FALSE;
static int mixer_fd = 0;

static int vol_set_vol (GtkAdjustment * adj, gpointer data);
static void vol_btn_enter (GtkWidget * widget, gpointer data);
static void vol_btn_leave (GtkWidget * widget, gpointer data);
static int popup_volume_window(GtkWidget *widget, GdkEventButton *event, gpointer user_data);

static void do_exit()
{
    gtk_main_quit();
}                                                                                                                            

static gboolean destroy_vol_popup ()
{
        if (is_inside_vol == FALSE) {
		gtk_widget_destroy(vol_wnd);
		vol_wnd = NULL;
        
		return FALSE;
        }
        return TRUE;
} 

/*
 *  Call an ioctl to change the master OSS volume.
 *  Stolen from GnomeRadio :)
 */
static int vol_set_vol (GtkAdjustment * adj, gpointer data)
{
        int i_vol;

        if (mixer_fd < 0)
                return -1;
                
        i_vol = (gint) adj->value;
        i_vol += (gint) adj->value << 8;

        if ((ioctl (mixer_fd, SOUND_MIXER_WRITE_VOLUME, &i_vol)) < 0)
                return 0;


  	return TRUE;
}

static void
vol_btn_enter (GtkWidget * widget, gpointer data)
{
        is_inside_vol = TRUE;
}
 
static void
vol_btn_leave (GtkWidget * widget, gpointer data)
{
        is_inside_vol = FALSE;
}

const gchar *sux_name (void)
{
	return "OSS Mixer";
}

int main (int argc, char *argv[])
{
    gtk_init (&argc, &argv);
    if (argc == 2)
	{
	    xid = atol(argv[1]);
	    window = gtk_plug_new(xid);
	    socket = gdk_window_foreign_new(xid);
	}
    else
	{
	    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	    gtk_window_set_title (GTK_WINDOW(window), "ROX-Mixer");
	}
    g_signal_connect(G_OBJECT (window),"destroy", G_CALLBACK (do_exit), NULL);


	mixer_fd = open("/dev/mixer", O_RDWR);

	if (mixer_fd > 0) {
		GtkWidget *button, *img;

		button = gtk_button_new();
		gtk_widget_show (button);
                
		gchar *icon_file=g_strconcat(getenv("APP_DIR"), "/.DirIcon", NULL);
		
		img = gtk_image_new_from_file(icon_file);

		g_free(icon_file);

		gtk_widget_show (img);
                gtk_container_add (GTK_CONTAINER (button), img);

                gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
                GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (button), GTK_CAN_FOCUS);

                g_signal_connect (G_OBJECT (button), "button_press_event",
                                           (GCallback) popup_volume_window, NULL);
                g_signal_connect (G_OBJECT (button), "enter_notify_event",
                                        (GCallback) vol_btn_enter, NULL);
                g_signal_connect (G_OBJECT (button), "leave_notify_event",
                                         (GCallback) vol_btn_leave, NULL);	
                                         
		gtk_container_add (GTK_CONTAINER(window), button); 
		gtk_widget_show (window);
		gtk_main ();
		                                                                                                 
	} else {
		GtkWidget *dialog;
                
                dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
                        GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                        "Can't open the mixer device.");
                gtk_dialog_run(GTK_DIALOG(dialog));
                gtk_widget_destroy(dialog);
	}

}


static int popup_volume_window(GtkWidget *widget, GdkEventButton *event,gpointer user_data)
{
    if (event->button == 1)
    {

        static GtkWidget *frame, *vscale;
        static GtkAdjustment *adj;
        static gint vol, x_pos, y_pos;
        GtkAllocation alloc;


        if (mixer_fd < 0 || vol_wnd != NULL)
                return;

	

        ioctl (mixer_fd, SOUND_MIXER_READ_VOLUME, &vol);

	
        vol_wnd = gtk_window_new (GTK_WINDOW_POPUP);

                 
        frame = gtk_frame_new (NULL);
        gtk_widget_show (frame);
        gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
        gtk_container_add (GTK_CONTAINER (vol_wnd), frame);

        adj = (GtkAdjustment*)gtk_adjustment_new (0, 0, 101, 1, 10, 1);
	
	
        gtk_adjustment_set_value (adj, vol >> 8);

        vscale = gtk_vscale_new (adj);
	gtk_range_set_inverted(GTK_RANGE(vscale),TRUE); 
        gtk_scale_set_digits (GTK_SCALE (vscale), 0);
        gtk_scale_set_draw_value (GTK_SCALE (vscale), FALSE);
        gtk_widget_set_usize(GTK_WIDGET(vscale),0,100);
        gtk_widget_show (vscale);
        gtk_container_add (GTK_CONTAINER (frame), vscale);

	gtk_widget_show(vol_wnd);

	alloc=GTK_WIDGET(vol_wnd)->allocation;
	x_pos = event->x_root;
	y_pos = event->y_root;
	if (y_pos>alloc.height) 
	    y_pos-=alloc.height;

	
	gtk_widget_set_uposition (GTK_WIDGET(vol_wnd),x_pos,y_pos);

        g_signal_connect (G_OBJECT (adj), "value_changed",
                            (GCallback)vol_set_vol, adj);


        g_signal_connect (G_OBJECT (vscale), "enter_notify_event",
                                   (GCallback) vol_btn_enter, NULL);
        g_signal_connect (G_OBJECT (vscale), "leave_notify_event",
                                   (GCallback) vol_btn_leave, NULL);

        g_timeout_add (2000, destroy_vol_popup, NULL);
	return 0;
    }
    return 1;
}

